function _random_poly(RR, n)
  pols = elem_type(RR)[]
  for t in 1:n
    f = MPolyBuildCtx(RR)
    g = MPolyBuildCtx(RR.R)
    for z in 1:3
      e = rand(0:6, ngens(RR.R))
      r = base_ring(RR.R)(rand(0:22))
      push_term!(f, r, e)
      push_term!(g, r, e)
    end
    f = finish(f)
    @test f == RR(finish(g))
    push!(pols, RR(finish(g)))
  end
  return pols
end

function _homogenous_polys(polys::Vector{<:MPolyElem})
  R = parent(polys[1])
  D = []
  _monomials = [zero(R)]
  for i = 1:4
    push!(D, homogenous_components(polys[i]))
    _monomials = append!(_monomials, monomials(polys[i]))
  end
  hom_polys = []
  for deg in unique([degree(mon) for mon = _monomials])
    g = zero(R)
    for i = 1:4
      if haskey(D[i], deg)
        temp = get(D[i], deg, 'x')
        @test homogenous_component(polys[i], deg) == temp
        g += temp
      end
    end
    @test ishomogenous(g)
    push!(hom_polys, g)
  end
  return hom_polys
end

@testset "mpoly-graded" begin

    Qx, (x,y,z) = PolynomialRing(QQ, ["x", "y", "z"])
    t = gen(Hecke.Globals.Qx)
    k1 , l= number_field(t^5+t^2+2)
    NFx = PolynomialRing(k1, ["x", "y", "z"])[1]
    k2 = Nemo.GF(23)
    GFx = PolynomialRing(k2, ["x", "y", "z"])[1]
    RNmodx=PolynomialRing(Nemo.ResidueRing(ZZ,17), :x => 1:2)[1]
    Rings= [Qx, NFx, GFx, RNmodx]

    A = abelian_group([4 3 0 1;
                       0 0 0 3])

    GrpElems = [A(fmpz[1,0,2,1]), A(fmpz[1,1,0,0]), A(fmpz[2,2,1,0])]

    T = grade(Qx)
    @test sprint(show, "text/plain", T(x)//T(1)) isa String

    v = [1,1,2]

    for R in Rings
      decorated_rings = [decorate(R),
                         grade(R, [v[i] for i=1:ngens(R)]),
                         filtrate(R, [v[i] for i=1:ngens(R)]),
                         filtrate(R, [GrpElems[i] for i =1:ngens(R)], (x, y) -> x[1]+x[2]+x[3]+x[4] < y[1]+y[2]+y[3]+y[4]),
                         grade(R, [GrpElems[i] for i =1:ngens(R)])]
    end

    for R in Rings
      decorated_rings = [decorate(R),
                         grade(R, [v[i] for i=1:ngens(R)]),
                         filtrate(R, [v[i] for i=1:ngens(R)]),
                         filtrate(R, [GrpElems[i] for i =1:ngens(R)], (x, y) -> x[1]+x[2]+x[3]+x[4] < y[1]+y[2]+y[3]+y[4]),
                         grade(R, [GrpElems[i] for i =1:ngens(R)])]

      graded_rings = decorated_rings[[2, 5]]
      filtered_rings =  decorated_rings[[1, 3, 4]]

      d_Elems = IdDict(decorated_rings[1] => 4*decorated_rings[1].D[1],
                       decorated_rings[2] => 5*decorated_rings[2].D[1],
                       decorated_rings[3] => 3*decorated_rings[3].D[1],
                       decorated_rings[4] => decorated_rings[4].D([2,2,1,0]),
                       decorated_rings[5] => decorated_rings[5].D([2,2,1,0]))

      dims = IdDict(zip(decorated_rings, [15, 12, 6, 1, 1]))

      # In RR, we take the quotient by the homogenous component corresponding to d_Elems[RR]
      # The dimension should be dims[RR]

      for RR in decorated_rings
        @test (RR in graded_rings) == Oscar.isgraded(RR)
        @test (RR in filtered_rings) == Oscar.isfiltered(RR)
        polys = _random_poly(RR, 4) # create 4 random polynomials
        @test ngens(RR) == length(gens(RR))
        @test gen(RR, 1) == RR[1]

        @test one(RR) == RR(1)
        @test isone(one(RR))
        @test zero(RR) == RR(0)
        @test iszero(zero(RR))
        @test !iszero(one(RR))
        @test !isone(zero(RR))
        @test divexact(one(RR), one(RR)) == one(RR)

        @test (polys[1] + polys[2])^2 == polys[1]^2 + 2*polys[1]*polys[2] + polys[2]^2
        @test (polys[3] - polys[4])^2 == polys[3]^2 + 2*(-polys[3])*polys[4] + polys[4]^2
        @test polys[2] * (polys[3] + polys[4]) == Oscar.addeq!(Oscar.mul!(polys[1], polys[2], polys[3]), Oscar.mul!(polys[1], polys[2], polys[4]))
        @test parent(polys[1]) === RR

        for k in 1:length(polys[4])
          @test coeff(polys[4],k) * Oscar.monomial(polys[4], k) ==
                finish(push_term!(MPolyBuildCtx(RR), collect(Oscar.MPolyCoeffs(polys[4]))[k], collect(Oscar.MPolyExponentVectors(polys[4]))[k]))
        end

        hom_polys = _homogenous_polys(polys)
        I = ideal(hom_polys)
        R_quo = Oscar.MPolyQuo(RR, I)
        @test base_ring(R_quo) == RR
        @test modulus(R_quo) == I
        f = R_quo(polys[2])
        D = homogenous_components(f)
        for deg in [degree(R_quo(mon)) for mon  = Oscar.monomials(f.f)]
          h = get(D, deg, 'x')
          @test ishomogenous(R_quo(h))
          @test h == homogenous_component(f, deg)
        end

        @test Oscar.isfiltered(R_quo) == Oscar.isfiltered(RR)
        @test Oscar.isgraded(R_quo) == Oscar.isgraded(RR)

        @test decoration(R_quo) == decoration(RR)

        d_Elem = d_Elems[RR]

        dim_test = dims[RR]

        if base_ring(R) isa AbstractAlgebra.Field
          grp_elem = d_Elem
          H = homogenous_component(RR, grp_elem)
          @test Oscar.hasrelshp(H[1], RR) !== nothing
          for g in gens(H[1])
            @test degree(H[2](g)) == grp_elem
            @test (H[2].g)(RR(g)) == g
          end
          @test dim(H[1]) == dim_test # j
        end
        #H_quo = homogenous_component(R_quo, grp_elem)
        #Oscar.hasrelshp(H_quo[1], R_quo) !== nothing
        #for g in gens(H_quo[1])
        #    degree(H_quo[2](g)) == grp_elem
        #    (H_quo[2].g)(R_quo(g)) == g
        #end
    end
  end
end
